"""Tests on isatab.py package"""
from __future__ import absolute_import
import io
import os
import sys
import unittest

from isatools import isatab
from isatools.model import *


class InvestigationParserUnitTests(unittest.TestCase):

    def setUp(self):
        self.parser = isatab.InvestigationParser()
        self.ontology_source_ref_section = \
            u'ONTOLOGY SOURCE REFERENCE\n' \
            u'Term Source Name\t"A"\t"B"\n' \
            u'Term Source File\t"f1"\t"f2"\n' \
            u'Term Source Version\t"1"\t"2"\n' \
            u'Term Source Description\t"d1"\t"d2"'

        self.os1 = OntologySource('A', 'f1', '1', 'd1')
        self.os2 = OntologySource('B', 'f2', '2', 'd2')

        self.investigation_section = \
            u'INVESTIGATION\n' \
            u'Investigation Identifier\t"ID"\n' \
            u'Investigation Title\t"T"\n' \
            u'Investigation Description\t"D"\n' \
            u'Investigation Submission Date\t"2017-10-24"\n' \
            u'Investigation Public Release Date\t"2017-10-25"'
        self.i = Investigation(identifier='ID', title='T', description='D',
                               submission_date='2017-10-24',
                               public_release_date='2017-10-25')

        self.investigation_section_with_comment_lines = \
            u'# This line is a comment\n' \
            u'INVESTIGATION\n' \
            u'Investigation Identifier\t"ID"\n' \
            u'Investigation Title\t"T"\n' \
            u'# This line is another comment\n' \
            u'Investigation Description\t"D"\n' \
            u'Investigation Submission Date\t"2017-10-24"\n' \
            u'Investigation Public Release Date\t"2017-10-25"'

        self.investigation_publications_section = \
            u'INVESTIGATION PUBLICATIONS\n' \
            u'Investigation PubMed ID\t"0"\t"1"\n' \
            u'Investigation Publication DOI\t"doi1"\t"doi2"\n' \
            u'Investigation Publication Author List\t"AL1"\t"AL2"\n' \
            u'Investigation Publication Title\t"T1"\t"T2"\n' \
            u'Investigation Publication Status\t"S1"\t"S2"\n' \
            u'Investigation Publication Status Term Accession Number\t"acc1"\t"acc2"\n' \
            u'Investigation Publication Status Term Source REF\t"src1"\t"src2"'

        publication_status1 = OntologyAnnotation('S1', 'src1', 'acc1')
        publication_status2 = OntologyAnnotation('S2', 'src2', 'acc2')
        self.publication1 = Publication('0', 'doi1', 'AL1', 'T1',
                                        publication_status1)
        self.publication2 = Publication('1', 'doi2', 'AL2', 'T2',
                                        publication_status2)

        self.study_publications_section = \
            u'STUDY\n' \
            u'STUDY PUBLICATIONS\n' \
            u'Study PubMed ID\t"0"\n' \
            u'Study Publication DOI\t"doi1"\n' \
            u'Study Publication Author List\t"AL1"\n' \
            u'Study Publication Title\t"T1"\n' \
            u'Study Publication Status\t"S1"\n' \
            u'Study Publication Status Term Accession Number\t"acc1"\n' \
            u'Study Publication Status Term Source REF\t"src1"\n' \
            u'STUDY\n' \
            u'STUDY PUBLICATIONS\n' \
            u'Study PubMed ID\t"0b"\t"1b"\n' \
            u'Study Publication DOI\t"doi1b"\t"doi2b"\n' \
            u'Study Publication Author List\t"AL1b"\t"AL2b"\n' \
            u'Study Publication Title\t"T1b"\t"T2b"\n' \
            u'Study Publication Status\t"S1b"\t"S2b"\n' \
            u'Study Publication Status Term Accession Number\t"acc1b"\t"acc2b"\n' \
            u'Study Publication Status Term Source REF\t"src1b"\t"src2b"'

        publication_status1b = OntologyAnnotation('S1b', 'src1b', 'acc1b')
        publication_status2b = OntologyAnnotation('S2b', 'src2b', 'acc2b')
        self.publication1b = Publication('0b', 'doi1b', 'AL1b', 'T1b',
                                         publication_status1b)
        self.publication2b = Publication('1b', 'doi2b', 'AL2b', 'T2b',
                                         publication_status2b)

        self.investigation_contacts_section = \
            u'INVESTIGATION CONTACTS\n' \
            u'Investigation Person Last Name\t"ln1"\t"ln2"\n' \
            u'Investigation Person First Name\t"fn1"\t"fn2"\n' \
            u'Investigation Person Mid Initials\t"mi1"\t"mi2"\n' \
            u'Investigation Person Email\t"e1"\t"e2"\n' \
            u'Investigation Person Phone\t"ph1"\t"ph2"\n' \
            u'Investigation Person Fax\t"fax1"\t"fax2"\n' \
            u'Investigation Person Address\t"ad1"\t"ad2"\n' \
            u'Investigation Person Affiliation\t"af1"\t"af2"\n' \
            u'Investigation Person Roles\t"r1"\t"r2"\n' \
            u'Investigation Person Roles Term Accession Number\t"r_acc1"\t"r_acc2"\n' \
            u'Investigation Person Roles Term Source REF\t"r_src1"\t"r_src2"'
        
        self.study_contacts_section = \
            u'STUDY\n'\
            u'STUDY CONTACTS\n' \
            u'Study Person Last Name\t"ln1"\t"ln2"\n' \
            u'Study Person First Name\t"fn1"\t"fn2"\n' \
            u'Study Person Mid Initials\t"mi1"\t"mi2"\n' \
            u'Study Person Email\t"e1"\t"e2"\n' \
            u'Study Person Phone\t"ph1"\t"ph2"\n' \
            u'Study Person Fax\t"fax1"\t"fax2"\n' \
            u'Study Person Address\t"ad1"\t"ad2"\n' \
            u'Study Person Affiliation\t"af1"\t"af2"\n' \
            u'Study Person Roles\t"r1"\t"r2"\n' \
            u'Study Person Roles Term Accession Number\t"r_acc1"\t"r_acc2"\n' \
            u'Study Person Roles Term Source REF\t"r_src1"\t"r_src2"\n' \
            u'STUDY\n' \
            u'STUDY CONTACTS\n' \
            u'Study Person Last Name\t"ln1b"\t"ln2b"\n' \
            u'Study Person First Name\t"fn1b"\t"fn2b"\n' \
            u'Study Person Mid Initials\t"mi1b"\t"mi2b"\n' \
            u'Study Person Email\t"e1b"\t"e2b"\n' \
            u'Study Person Phone\t"ph1b"\t"ph2b"\n' \
            u'Study Person Fax\t"fax1b"\t"fax2b"\n' \
            u'Study Person Address\t"ad1b"\t"ad2b"\n' \
            u'Study Person Affiliation\t"af1b"\t"af2b"\n' \
            u'Study Person Roles\t"r1b"\t"r2b"\n' \
            u'Study Person Roles Term Accession Number\t"r_acc1b"\t"r_acc2b"\n' \
            u'Study Person Roles Term Source REF\t"r_src1b"\t"r_src2b"'

        role1 = OntologyAnnotation('r1', 'r_src1', 'r_acc1')
        role2 = OntologyAnnotation('r2', 'r_src2', 'r_acc2')
        self.person1 = Person('ln1', 'fn1', 'mi1', 'e1', 'ph1', 'fax1', 'ad1',
                              'af1', [role1])
        self.person2 = Person('ln2', 'fn2', 'mi2', 'e2', 'ph2', 'fax2', 'ad2', 
                              'af2', [role2])
        role1b = OntologyAnnotation('r1b', 'r_src1b', 'r_acc1b')
        role2b = OntologyAnnotation('r2b', 'r_src2b', 'r_acc2b')
        self.person1b = Person('ln1b', 'fn1b', 'mi1b', 'e1b', 'ph1b', 'fax1b', 
                               'ad1b', 'af1b', [role1b])
        self.person2b = Person('ln2b', 'fn2b', 'mi2b', 'e2b', 'ph2b', 'fax2b', 
                               'ad2b', 'af2b', [role2b])

        self.study_section = \
            u'STUDY\n' \
            u'Study Identifier\t"ID"\n' \
            u'Study Title\t"T"\n' \
            u'Study Description\t"D"\n' \
            u'Study Submission Date\t"2017-10-24"\n' \
            u'Study Public Release Date\t"2017-10-25"\n' \
            u'Study File Name\t"f"'

        self.s = Study(identifier='ID', title='T', description='D',
                               submission_date='2017-10-24',
                               public_release_date='2017-10-25', filename='f')

        self.study_design_descriptors_section = \
            u'STUDY\n' \
            u'STUDY DESIGN DESCRIPTORS\n' \
            u'Study Design Type\t"d1"\t"d2"\n' \
            u'Study Design Type Term Accession Number\t"acc1"\t"acc2"\n' \
            u'Study Design Type Term Source REF\t"src1"\t"src2"'

        self.study_design = [
            OntologyAnnotation('d1', 'src1', 'acc1'),
            OntologyAnnotation('d2', 'src2', 'acc2')
        ]

        self.study_factors_section = \
            u'STUDY\n' \
            u'STUDY FACTORS\n' \
            u'Study Factor Name\t"f1"\t"f2"\n' \
            u'Study Factor Type\t"t1"\t"t2"\n' \
            u'Study Factor Type Term Accession Number\t"acc1"\t"acc2"\n' \
            u'Study Factor Type Term Source REF\t"src1"\t"src2"'

        self.study_factors = [
            StudyFactor(name='f1',
                        factor_type=OntologyAnnotation('t1', 'src1', 'acc1')),
            StudyFactor(name='f2',
                        factor_type=OntologyAnnotation('t2', 'src2', 'acc2'))
        ]
        
        self.study_assays_section = \
            u'STUDY\n' \
            u'STUDY ASSAYS\n' \
            u'Study Assay Measurement Type\t"mt1"\t"mt2"\n' \
            u'Study Assay Measurement Type Term Accession Number\t"mt_acc1"\t"mt_acc2"\n' \
            u'Study Assay Measurement Type Term Source REF\t"mt_src1"\t"mt_src2"\n' \
            u'Study Assay Technology Type\t"tt1"\t"tt2"\n' \
            u'Study Assay Technology Type Term Accession Number\t"tt_acc1"\t"tt_acc2"\n' \
            u'Study Assay Technology Type Term Source REF\t"tt_src1"\t"tt_src2"\n' \
            u'Study Assay Technology Platform\t"tp1"\t"tp2"\n' \
            u'Study Assay File Name\t"f1"\t"f2"'
        
        self.study_assay1 = Assay(
            measurement_type=OntologyAnnotation('mt1', 'mt_src1', 'mt_acc1'),
            technology_type=OntologyAnnotation('tt1', 'tt_src1', 'tt_acc1'),
            technology_platform='tp1', filename='f1'
        )
        self.study_assay2 = Assay(
            measurement_type=OntologyAnnotation('mt2', 'mt_src2', 'mt_acc2'),
            technology_type=OntologyAnnotation('tt2', 'tt_src2', 'tt_acc2'),
            technology_platform='tp2', filename='f2'
        )

        self.study_protocols_section = \
            u'STUDY\n' \
            u'STUDY PROTOCOLS\n' \
            u'Study Protocol Name\t"p1"\t"p2"\n' \
            u'Study Protocol Type\t"pt1"\t"pt2"\n' \
            u'Study Protocol Type Term Accession Number\t"pt1_acc"\t"pt2_acc"\n' \
            u'Study Protocol Type Term Source REF\t"pt1_src"\t"pt2_src"\n' \
            u'Study Protocol Description\t"d1"\t"d2"\n' \
            u'Study Protocol URI\t"u1"\t"u2"\n' \
            u'Study Protocol Version\t"v1"\t"v2"\n' \
            u'Study Protocol Parameters Name\t"ppn1a;ppn1b"\t"ppn2"\n' \
            u'Study Protocol Parameters Name Term Accession Number\t"ppn1a_acc;ppn1b_acc"\t"ppn2_acc"\n' \
            u'Study Protocol Parameters Name Term Source REF\t"ppn1a_src;ppn1b_src"\t"ppn2_src"\n' \
            u'Study Protocol Components Name\t"cn1a;cn1b"\t"cn2"\n' \
            u'Study Protocol Components Type\t"ct1a;ct1b"\t"ct2"\n' \
            u'Study Protocol Components Type Term Accession Number\t"ct1a_acc;ct1b_acc"\t"ct2_acc"\n' \
            u'Study Protocol Components Type Term Source REF\t"ct1a_src;ct1b_src"\t"ct2_src"\n' \
            u'STUDY\n' \
            u'STUDY PROTOCOLS\n' \
            u'Study Protocol Name\t"p1b"\n' \
            u'Study Protocol Type\t"pt1b"\n' \
            u'Study Protocol Type Term Accession Number\t"pt1b_acc"\n' \
            u'Study Protocol Type Term Source REF\t"pt1b_src"\n' \
            u'Study Protocol Description\t"d1b"\n' \
            u'Study Protocol URI\t"u1b"\n' \
            u'Study Protocol Version\t"v1b"\n' \
            u'Study Protocol Parameters Name\t"ppn1b"\n' \
            u'Study Protocol Parameters Name Term Accession Number\t"ppn1b_acc"\n' \
            u'Study Protocol Parameters Name Term Source REF\t"ppn1b_src"\n' \
            u'Study Protocol Components Name\t"cn1b"\n' \
            u'Study Protocol Components Type\t"ct1b"\n' \
            u'Study Protocol Components Type Term Accession Number\t"ct1b_acc"\n' \
            u'Study Protocol Components Type Term Source REF\t"ct1b_src"'

        self.study_protocol1 = Protocol(
            '', 'p1', OntologyAnnotation('pt1', 'pt1_src', 'pt1_acc'), 'u1',
            'd1', 'v1', [
                ProtocolParameter(
                    parameter_name=OntologyAnnotation('ppn1a', 'ppn1a_src',
                                                      'ppn1a_acc')),
                ProtocolParameter(
                    parameter_name=OntologyAnnotation('ppn1b', 'ppn1b_src',
                                                      'ppn1b_acc'))
            ], [
                ProtocolComponent(
                    name='cn1a',
                    component_type=OntologyAnnotation('ct1a', 'ct1a_src',
                                                      'ct1a_acc')),
                ProtocolComponent(
                    name='cn1b',
                    component_type=OntologyAnnotation('ct1b', 'ct1b_src',
                                                      'ct1b_acc'))
            ])
        self.study_protocol2 = Protocol(
            '', 'p2', OntologyAnnotation('pt2', 'pt2_src', 'pt2_acc'), 'u2',
            'd2', 'v2', [
                ProtocolParameter(
                    parameter_name=OntologyAnnotation('ppn2', 'ppn2_src',
                                                      'ppn2_acc'))
            ], [
                ProtocolComponent(
                    name='cn2',
                    component_type=OntologyAnnotation('ct2', 'ct2_src',
                                                      'ct2_acc'))
            ])
        self.study_protocol1b = Protocol(
            '', 'p1b', OntologyAnnotation('pt1b', 'pt1b_src', 'pt1b_acc'),
            'u1b',
            'd1b', 'v1b', [
                ProtocolParameter(
                    parameter_name=OntologyAnnotation('ppn1b', 'ppn1b_src',
                                                      'ppn1b_acc'))
            ], [
                ProtocolComponent(
                    name='cn1b',
                    component_type=OntologyAnnotation('ct1b', 'ct1b_src',
                                                      'ct1b_acc'))
            ])

    def test_parse_and_ignore_comment_lines(self):
        self.parser.parse(
            io.StringIO(self.investigation_section_with_comment_lines))
        self.assertEqual(self.i, self.parser.isa)
        
    def test_parse_ontology_source_reference(self):
        self.parser.parse(io.StringIO(self.ontology_source_ref_section))
        self.assertIn(self.os1, self.parser.isa.ontology_source_references)
        self.assertIn(self.os2, self.parser.isa.ontology_source_references)

    def test_parse_investigation_section(self):
        self.parser.parse(io.StringIO(self.investigation_section))
        self.assertEqual(self.i, self.parser.isa)

    def test_parse_investigation_publications_section(self):
        self.parser.parse(io.StringIO(self.investigation_publications_section))
        self.assertIn(self.publication1, self.parser.isa.publications)

    def test_parse_study_publications_section(self):
        self.parser.parse(io.StringIO(self.study_publications_section))
        self.assertIn(self.publication1b,
                      self.parser.isa.studies[-1].publications)
        self.assertIn(self.publication2b,
                      self.parser.isa.studies[-1].publications)
        
    def test_parse_investigation_contacts_section(self):
        self.parser.parse(io.StringIO(self.investigation_contacts_section))
        self.assertIn(self.person1, self.parser.isa.contacts)

    def test_parse_study_contacts_section(self):
        self.parser.parse(io.StringIO(self.study_contacts_section))
        self.assertIn(self.person1b,
                      self.parser.isa.studies[-1].contacts)
        self.assertIn(self.person2b,
                      self.parser.isa.studies[-1].contacts)

    def test_parse_study_section(self):
        self.parser.parse(io.StringIO(self.study_section))
        self.assertEqual(self.s, self.parser.isa.studies[-1])

    def test_parse_study_design_descriptors_section(self):
        self.parser.parse(io.StringIO(self.study_design_descriptors_section))
        self.assertEqual(self.study_design,
                         self.parser.isa.studies[-1].design_descriptors)

    def test_parse_study_factors_section(self):
        self.parser.parse(io.StringIO(self.study_factors_section))
        self.assertEqual(self.study_factors,
                         self.parser.isa.studies[-1].factors)

    def test_parse_study_assays_section(self):
        self.parser.parse(io.StringIO(self.study_assays_section))
        self.assertIn(self.study_assay1, self.parser.isa.studies[-1].assays)
        self.assertIn(self.study_assay2, self.parser.isa.studies[-1].assays)

    def test_parse_study_protocols_section(self):
        self.parser.parse(io.StringIO(self.study_protocols_section))
        self.assertIn(self.study_protocol1,
                      self.parser.isa.studies[0].protocols)
        self.assertIn(self.study_protocol2,
                      self.parser.isa.studies[0].protocols)
        self.assertIn(self.study_protocol1b,
                      self.parser.isa.studies[1].protocols)


class InvestigationParserIntegrationTests(unittest.TestCase):

    def setUp(self):
        self._tab_data_dir = os.path.join(
            os.path.dirname(__file__), 'data', 'tab')
        self.parser = isatab.InvestigationParser()

    def tearDown(self):
        pass

    def test_isatab_load_bii_i_1(self):
        with io.open(os.path.join(self._tab_data_dir, 'BII-I-1',
                                  'i_investigation.txt')) as fp:
            self.parser.parse(fp)
            isa = self.parser.isa
            self.assertListEqual([s.filename for s in isa.studies],
                                 ['s_BII-S-1.txt',
                                  's_BII-S-2.txt'])
            study_bii_s_1 = \
            [s for s in isa.studies if s.filename == 's_BII-S-1.txt'][0]
            self.assertListEqual([a.filename for a in study_bii_s_1.assays],
                                 ['a_proteome.txt', 'a_metabolome.txt',
                                  'a_transcriptome.txt'])
            study_bii_s_2 = \
            [s for s in isa.studies if s.filename == 's_BII-S-2.txt'][0]
            self.assertListEqual([a.filename for a in study_bii_s_2.assays],
                                 ['a_microarray.txt'])

    def test_isatab_load_bii_s_3(self):
        with io.open(os.path.join(self._tab_data_dir, 'BII-S-3',
                                  'i_gilbert.txt')) as fp:
            self.parser.parse(fp)
            isa = self.parser.isa

            self.assertListEqual([s.filename for s in isa.studies],
                                 ['s_BII-S-3.txt'])
            study_bii_s_3 = \
            [s for s in isa.studies if s.filename == 's_BII-S-3.txt'][0]
            self.assertListEqual([a.filename for a in study_bii_s_3.assays],
                                 ['a_gilbert-assay-Gx.txt',
                                  'a_gilbert-assay-Tx.txt'])

    def test_isatab_load_bii_s_7(self):
        with io.open(os.path.join(self._tab_data_dir, 'BII-S-7',
                                  'i_matteo.txt')) as fp:
            self.parser.parse(fp)
            isa = self.parser.isa

            self.assertListEqual([s.filename for s in isa.studies], [
                's_BII-S-7.txt'])
            study_bii_s_7 = \
            [s for s in isa.studies if s.filename == 's_BII-S-7.txt'][0]
            self.assertListEqual([a.filename for a in study_bii_s_7.assays],
                                 ['a_matteo-assay-Gx.txt'])

def suite():
    s = unittest.TestLoader().loadTestsFromTestCase(InvestigationParserUnitTests)
    return s

def main():
    result = unittest.TextTestRunner(verbosity=2).run(suite())
    return 0 if result.wasSuccessful() else 1

if __name__ == '__main__':
    sys.exit(main())



"""Tests on isajson.py package"""
from __future__ import absolute_import
import json
import unittest
import os

from isatools import isajson


class TestIsaJson(unittest.TestCase):

    def setUp(self):
        self._json_data_dir = os.path.join(os.path.dirname(__file__), 'data',
                                           'json')

    def test_json_load_and_dump_bii_i_1(self):
        with open(os.path.join(self._json_data_dir, 'BII-I-1',
                               'BII-I-1.json')) as isajson_fp:
            ISA = isajson.load(isajson_fp)
            ISA_J = json.loads(json.dumps(ISA, cls=isajson.ISAJSONEncoder))
            self.assertListEqual([s['filename'] for s in ISA_J['studies']],
                                 ['s_BII-S-1.txt', 's_BII-S-2.txt'])
            study_bii_s_1 = \
            [s for s in ISA_J['studies'] if s['filename'] == 's_BII-S-1.txt'][0]
            self.assertEqual(len(study_bii_s_1['materials']['sources']), 18)
            self.assertEqual(len(study_bii_s_1['materials']['samples']), 164)
            self.assertEqual(len(study_bii_s_1['processSequence']), 18)
            self.assertListEqual(
                [a['filename'] for a in study_bii_s_1['assays']],
                ['a_proteome.txt', 'a_metabolome.txt',
                 'a_transcriptome.txt'])
            assay_proteome = [a for a in study_bii_s_1['assays'] if
                              a['filename'] == 'a_proteome.txt'][0]

            self.assertEqual(len(assay_proteome['materials']['samples']), 8)
            self.assertEqual(len(assay_proteome['materials']['otherMaterials']),
                             19)
            self.assertEqual(len(assay_proteome['dataFiles']), 7)
            self.assertEqual(len(assay_proteome['processSequence']), 25)

            assay_metabolome = [a for a in study_bii_s_1['assays'] if
                                a['filename'] == 'a_metabolome.txt'][0]
            self.assertEqual(len(assay_metabolome['materials']['samples']), 92)
            self.assertEqual(
                len(assay_metabolome['materials']['otherMaterials']), 92)
            self.assertEqual(len(assay_metabolome['dataFiles']), 111)
            self.assertEqual(len(assay_metabolome['processSequence']), 203)

            assay_transcriptome = [a for a in study_bii_s_1['assays'] if
                                   a['filename'] == 'a_transcriptome.txt'][0]
            self.assertEqual(len(assay_transcriptome['materials']['samples']),
                             48)
            self.assertEqual(
                len(assay_transcriptome['materials']['otherMaterials']), 96)
            self.assertEqual(len(assay_transcriptome['dataFiles']), 49)
            self.assertEqual(len(assay_transcriptome['processSequence']), 193)
            study_bii_s_2 = \
            [s for s in ISA_J['studies'] if s['filename'] == 's_BII-S-2.txt'][0]
            self.assertEqual(len(study_bii_s_2['materials']['sources']), 1)
            self.assertEqual(len(study_bii_s_2['materials']['samples']), 2)
            self.assertEqual(len(study_bii_s_2['processSequence']), 1)
            self.assertEqual(len(study_bii_s_2['assays']), 1)
            self.assertListEqual(
                [a['filename'] for a in study_bii_s_2['assays']],
                ['a_microarray.txt'])
            assay_microarray = [a for a in study_bii_s_2['assays'] if
                                a['filename'] == 'a_microarray.txt'][0]

            self.assertEqual(len(assay_microarray['materials']['samples']), 2)
            self.assertEqual(
                len(assay_microarray['materials']['otherMaterials']), 28)
            self.assertEqual(len(assay_microarray['dataFiles']), 15)
            self.assertEqual(len(assay_microarray['processSequence']), 45)

    def test_json_load_and_dump_bii_s_3(self):
        with open(os.path.join(self._json_data_dir, 'BII-S-3',
                               'BII-S-3.json')) as isajson_fp:
            ISA = isajson.load(isajson_fp)
            ISA_J = json.loads(json.dumps(ISA, cls=isajson.ISAJSONEncoder))
            self.assertListEqual([s['filename'] for s in ISA_J['studies']],
                                 ['s_BII-S-3.txt'])
            study_bii_s_3 = \
            [s for s in ISA_J['studies'] if s['filename'] == 's_BII-S-3.txt'][0]
            self.assertEqual(len(study_bii_s_3['materials']['sources']), 4)
            self.assertEqual(len(study_bii_s_3['materials']['samples']), 4)
            self.assertEqual(len(study_bii_s_3['processSequence']), 4)
            self.assertListEqual(
                [a['filename'] for a in study_bii_s_3['assays']],
                ['a_gilbert-assay-Gx.txt', 'a_gilbert-assay-Tx.txt'])
            assay_gx = [a for a in study_bii_s_3['assays'] if
                        a['filename'] == 'a_gilbert-assay-Gx.txt'][0]
            self.assertEqual(len(assay_gx['materials']['samples']), 4)
            self.assertEqual(len(assay_gx['materials']['otherMaterials']), 4)
            self.assertEqual(len(assay_gx['dataFiles']), 6)
            self.assertEqual(len(assay_gx['processSequence']), 18)
            assay_tx = [a for a in study_bii_s_3['assays'] if
                        a['filename'] == 'a_gilbert-assay-Tx.txt'][0]
            self.assertEqual(len(assay_tx['materials']['samples']), 4)
            self.assertEqual(len(assay_tx['materials']['otherMaterials']), 4)
            self.assertEqual(len(assay_tx['dataFiles']), 24)
            self.assertEqual(len(assay_tx['processSequence']), 36)

    def test_json_load_and_dump_bii_s_7(self):
        # Load into ISA objects
        with open(os.path.join(self._json_data_dir, 'BII-S-7',
                               'BII-S-7.json')) as isajson_fp:
            ISA = isajson.load(isajson_fp)
            ISA_J = json.loads(json.dumps(ISA, cls=isajson.ISAJSONEncoder))
            self.assertListEqual([s['filename'] for s in ISA_J['studies']],
                                 ['s_BII-S-7.txt'])
            study_bii_s_7 = \
            [s for s in ISA_J['studies'] if s['filename'] == 's_BII-S-7.txt'][0]
            self.assertEqual(len(study_bii_s_7['materials']['sources']), 29)
            self.assertEqual(len(study_bii_s_7['materials']['samples']), 29)
            self.assertEqual(len(study_bii_s_7['processSequence']), 29)
            self.assertListEqual(
                [a['filename'] for a in study_bii_s_7['assays']],
                ['a_matteo-assay-Gx.txt'])
            assay_gx = [a for a in study_bii_s_7['assays'] if
                        a['filename'] == 'a_matteo-assay-Gx.txt'][0]
            self.assertEqual(len(assay_gx['materials']['samples']), 29)
            self.assertEqual(len(assay_gx['materials']['otherMaterials']), 29)
            self.assertEqual(len(assay_gx['dataFiles']), 29)
            self.assertEqual(len(assay_gx['processSequence']), 116)


"""Tests on isatools.model classes"""
from __future__ import absolute_import
import datetime
import unittest

from isatools.model import *


class CommentTest(unittest.TestCase):

    def setUp(self):
        self.comment_default = Comment()
        self.comment = Comment(name='N', value='V')

    def test_repr(self):
        self.assertEqual("isatools.model.Comment(name='', value='')",
                         repr(self.comment_default))
        self.assertEqual("isatools.model.Comment(name='N', value='V')",
                         repr(self.comment))

    def test_str(self):
        self.assertEqual("""Comment(
    name=
    value=
)""", str(self.comment_default))

        self.assertEqual("""Comment(
    name=N
    value=V
)""", str(self.comment))

    def test_eq(self):
        expected_comment = Comment(name='N', value='V')

        self.assertEqual(expected_comment, self.comment)
        self.assertEqual(hash(expected_comment), hash(self.comment))

    def test_ne(self):
        expected_other_comment = Comment(name='V', value='N')

        self.assertNotEqual(expected_other_comment, self.comment)
        self.assertNotEqual(hash(expected_other_comment), hash(self.comment))

    def test_raises_ISAModelAttributeError(self):
        try:
            self.comment_default.name = 0
        except ISAModelAttributeError:
            pass
        except Exception:
            self.fail('ISAModelAttributeError not raised')

        try:
            self.comment_default.value = 0
        except ISAModelAttributeError:
            pass
        except Exception:
            self.fail('ISAModelAttributeError not raised')


class InvestigationTest(unittest.TestCase):

    def setUp(self):
        self.investigation_default = Investigation()
        self.investigation = Investigation(
            identifier='id', filename='file', title='T',
            submission_date=datetime.datetime(day=1, month=1, year=2017),
            public_release_date=datetime.datetime(day=1, month=1, year=2017))

    def test_repr(self):
        self.assertEqual("isatools.model.Investigation(identifier='', "
                         "filename='', title='', submission_date='', "
                         "public_release_date='', "
                         "ontology_source_references=[], publications=[], "
                         "contacts=[], studies=[], comments=[])",
                         repr(self.investigation_default))
        self.assertEqual("isatools.model.Investigation(identifier='id', "
                         "filename='file', title='T', "
                         "submission_date='2017-01-01 00:00:00', "
                         "public_release_date='2017-01-01 00:00:00', "
                         "ontology_source_references=[], publications=[], "
                         "contacts=[], studies=[], comments=[])",
                         repr(self.investigation))

    def test_str(self):
        self.assertEqual("""Investigation(
    identifier=
    filename=
    title=
    submission_date=
    public_release_date=
    ontology_source_references=0 OntologySource objects
    publications=0 Publication objects
    contacts=0 Person objects
    studies=0 Study objects
    comments=0 Comment objects
)""", str(self.investigation_default))

        self.assertEqual("""Investigation(
    identifier=id
    filename=file
    title=T
    submission_date=2017-01-01 00:00:00
    public_release_date=2017-01-01 00:00:00
    ontology_source_references=0 OntologySource objects
    publications=0 Publication objects
    contacts=0 Person objects
    studies=0 Study objects
    comments=0 Comment objects
)""", str(self.investigation))

    def test_eq(self):
        expected_investigation = Investigation(
            identifier='id', filename='file', title='T',
            submission_date=datetime.datetime(day=1, month=1, year=2017),
            public_release_date=datetime.datetime(day=1, month=1, year=2017))
        self.assertEqual(expected_investigation, self.investigation)
        self.assertEqual(hash(expected_investigation), hash(self.investigation))

    def test_ne(self):
        expected_other_investigation = Investigation(
            identifier='id2', filename='file2', title='T2',
            submission_date=datetime.datetime(day=2, month=1, year=2017),
            public_release_date=datetime.datetime(day=2, month=1, year=2017))
        self.assertNotEqual(expected_other_investigation, self.investigation)
        self.assertNotEqual(
            hash(expected_other_investigation), hash(self.investigation))


class OntologySourceTest(unittest.TestCase):

    def setUp(self):
        self.ontology_source_default = OntologySource(name='N')
        self.ontology_source = OntologySource(name='N', file='F', version='V',
                                              description='D')

    def test_repr(self):
        self.assertEqual("isatools.model.OntologySource(name='N', file='', "
                         "version='', description='', comments=[])",
                         repr(self.ontology_source_default))
        self.assertEqual("isatools.model.OntologySource(name='N', file='F', "
                         "version='V', description='D', comments=[])",
                         repr(self.ontology_source))

    def test_str(self):
        self.assertEqual("""OntologySource(
    name=N
    file=
    version=
    description=
    comments=0 Comment objects
)""", str(self.ontology_source_default))

        self.assertEqual("""OntologySource(
    name=N
    file=F
    version=V
    description=D
    comments=0 Comment objects
)""", str(self.ontology_source))

    def test_eq(self):
        expected_ontology_source = OntologySource(
            name='N', file='F', version='V', description='D', comments=[])
        self.assertEqual(expected_ontology_source, self.ontology_source)
        self.assertEqual(
            hash(expected_ontology_source),  hash(self.ontology_source))

    def test_ne(self):
        expected_other_ontology_source = OntologySource(
            name='N2', file='F2', version='V2', description='D2', comments=[])
        self.assertNotEqual(
            expected_other_ontology_source, self.ontology_source)
        self.assertNotEqual(
            hash(expected_other_ontology_source), hash(self.ontology_source))


class OntologyAnnotationTest(unittest.TestCase):

    def setUp(self):
        self.ontology_annotation_default = OntologyAnnotation()
        self.ontology_annotation = OntologyAnnotation(
            term='T', term_source=OntologySource('N'), term_accession='A')

    def test_repr(self):
        self.assertEqual("isatools.model.OntologyAnnotation(term='', "
                         "term_source=None, term_accession='', comments=[])",
                         repr(self.ontology_annotation_default))
        self.assertEqual("isatools.model.OntologyAnnotation(term='T', "
                         "term_source=isatools.model.OntologySource("
                         "name='N', file='', version='', description='', "
                         "comments=[]), term_accession='A', comments=[])",
                         repr(self.ontology_annotation))

    def test_str(self):
        self.assertEqual("""OntologyAnnotation(
    term=
    term_source=
    term_accession=
    comments=0 Comment objects
)""", str(self.ontology_annotation_default))

        self.assertEqual("""OntologyAnnotation(
    term=T
    term_source=N
    term_accession=A
    comments=0 Comment objects
)""", str(self.ontology_annotation))

    def test_eq(self):
        expected_ontology_annotation = OntologyAnnotation(
            term='T', term_source=OntologySource(name='N'), term_accession='A')
        self.assertEqual(expected_ontology_annotation, self.ontology_annotation)
        self.assertEqual(
            hash(expected_ontology_annotation),  hash(self.ontology_annotation))

    def test_ne(self):
        expected_other_ontology_annotation = OntologyAnnotation(
            term='T2', term_source=OntologySource(name='N2'),
            term_accession='A2')
        self.assertNotEqual(expected_other_ontology_annotation, self.ontology_annotation)
        self.assertNotEqual(
            hash(expected_other_ontology_annotation), hash(self.ontology_annotation))


class PublicationTest(unittest.TestCase):

    def setUp(self):
        self.publication_default = Publication()
        self.publication = Publication(
            pubmed_id='1', doi='10', author_list='A. Author', title='T',
            status=OntologyAnnotation(term='S'))

    def test_repr(self):
        self.assertEqual("isatools.model.Publication(pubmed_id='', "
                         "doi='', author_list='', title='', status=None, "
                         "comments=[])",
                         repr(self.publication_default))
        self.assertEqual("isatools.model.Publication(pubmed_id='1', "
                         "doi='10', author_list='A. Author', title='T', "
                         "status=isatools.model.OntologyAnnotation(term='S', "
                         "term_source=None, term_accession='', comments=[]), "
                         "comments=[])",
                         repr(self.publication))

    def test_str(self):
        self.assertEqual("""Publication(
    pubmed_id=
    doi=
    author_list=
    title=
    status=
    comments=0 Comment objects
)""", str(self.publication_default))

        self.assertEqual("""Publication(
    pubmed_id=1
    doi=10
    author_list=A. Author
    title=T
    status=S
    comments=0 Comment objects
)""", str(self.publication))

    def test_eq(self):
        expected_publication = Publication(
            pubmed_id='1', doi='10', author_list='A. Author', title='T', 
            status=OntologyAnnotation(term='S'))
        self.assertEqual(expected_publication, self.publication)
        self.assertEqual(
            hash(expected_publication),  hash(self.publication))

    def test_ne(self):
        expected_other_publication = Publication(
            pubmed_id='2', doi='20', author_list='B. Author', title='T2',
            status=OntologyAnnotation(term='S2'))
        self.assertNotEqual(expected_other_publication, self.publication)
        self.assertNotEqual(
            hash(expected_other_publication), hash(self.publication))


class PersonTest(unittest.TestCase):

    def setUp(self):
        self.person_default = Person()
        self.person = Person(last_name='L', first_name='F', mid_initials='M',
                             email='a@b.com', phone='0', fax='1', address='A',
                             affiliation='Af',
                             roles=[OntologyAnnotation(term='R')])

    def test_repr(self):
        self.assertEqual("isatools.model.Person(last_name='', first_name='', "
                         "mid_initials='', email='', phone='', fax='', "
                         "address='', affiliation='', roles=[], comments=[])",
                         repr(self.person_default))
        self.assertEqual("isatools.model.Person(last_name='L', first_name='F', "
                         "mid_initials='M', email='a@b.com', phone='0', "
                         "fax='1', address='A', affiliation='Af', "
                         "roles=[isatools.model.OntologyAnnotation(term='R', "
                         "term_source=None, term_accession='', comments=[])], "
                         "comments=[])",
                         repr(self.person))

    def test_str(self):
        self.assertEqual("""Person(
    last_name=
    first_name=
    mid_initials=
    email=
    phone=
    fax=
    address=
    roles=0 OntologyAnnotation objects
    comments=0 Comment objects
)""", str(self.person_default))

        self.assertEqual("""Person(
    last_name=L
    first_name=F
    mid_initials=M
    email=a@b.com
    phone=0
    fax=1
    address=A
    roles=1 OntologyAnnotation objects
    comments=0 Comment objects
)""", str(self.person))

    def test_eq(self):
        expected_person = Person(last_name='L', first_name='F', 
                                 mid_initials='M', email='a@b.com', phone='0', 
                                 fax='1', address='A', affiliation='Af', 
                                 roles=[OntologyAnnotation(
                                     term='R', term_source=None, 
                                     term_accession='', comments=[])], 
                                 comments=[])
        self.assertEqual(expected_person, self.person)
        self.assertEqual(hash(expected_person),  hash(self.person))

    def test_ne(self):
        expected_other_person = Person(last_name='F', first_name='L',
                                 mid_initials='M2', email='c@d.com', phone='1',
                                 fax='2', address='A2', affiliation='Af2',
                                 roles=[OntologyAnnotation(
                                     term='R2', term_source=None,
                                     term_accession='', comments=[])],
                                 comments=[])
        self.assertNotEqual(expected_other_person, self.person)
        self.assertNotEqual(
            hash(expected_other_person), hash(self.person))


class StudyTest(unittest.TestCase):

    def setUp(self):
        self.study_default = Study()
        self.study = Study(
            filename='file', identifier='0', title='T', description='D',
            submission_date=datetime.datetime(day=1, month=1, year=2017),
            public_release_date=datetime.datetime(day=1, month=1, year=2017))

    def test_repr(self):
        self.assertEqual("isatools.model.Study(filename='', "
                         "identifier='', title='', description='', "
                         "submission_date='', public_release_date='', "
                         "contacts=[], design_descriptors=[], publications=[], "
                         "factors=[], protocols=[], assays=[], sources=[], "
                         "samples=[], process_sequence=[], other_material=[], "
                         "characteristic_categories=[], comments=[], units=[])",
                         repr(self.study_default))
        self.assertEqual("isatools.model.Study(filename='file', "
                         "identifier='0', title='T', description='D', "
                         "submission_date='2017-01-01 00:00:00', "
                         "public_release_date='2017-01-01 00:00:00', "
                         "contacts=[], design_descriptors=[], publications=[], "
                         "factors=[], protocols=[], assays=[], sources=[], "
                         "samples=[], process_sequence=[], other_material=[], "
                         "characteristic_categories=[], comments=[], units=[])",
                         repr(self.study))

    def test_str(self):
        self.assertEqual("""Study(
    identifier=
    filename=
    title=
    description=
    submission_date=
    public_release_date=
    contacts=0 Person objects
    design_descriptors=0 OntologyAnnotation objects
    publications=0 Publication objects
    factors=0 StudyFactor objects
    protocols=0 Protocol objects
    assays=0 Assay objects
    sources=0 Source objects
    samples=0 Sample objects
    process_sequence=0 Process objects
    other_material=0 Material objects
    characteristic_categories=0 OntologyAnnotation objects
    comments=0 Comment objects
    units=0 Unit objects
)""", str(self.study_default))

        self.assertEqual("""Study(
    identifier=0
    filename=file
    title=T
    description=D
    submission_date=2017-01-01 00:00:00
    public_release_date=2017-01-01 00:00:00
    contacts=0 Person objects
    design_descriptors=0 OntologyAnnotation objects
    publications=0 Publication objects
    factors=0 StudyFactor objects
    protocols=0 Protocol objects
    assays=0 Assay objects
    sources=0 Source objects
    samples=0 Sample objects
    process_sequence=0 Process objects
    other_material=0 Material objects
    characteristic_categories=0 OntologyAnnotation objects
    comments=0 Comment objects
    units=0 Unit objects
)""", str(self.study))

    def test_eq(self):
        expected_study = Study(
            filename='file', identifier='0', title='T', description='D',
            submission_date=datetime.datetime(day=1, month=1, year=2017),
            public_release_date=datetime.datetime(day=1, month=1, year=2017),
            contacts=[], design_descriptors=[], publications=[], factors=[],
            protocols=[], assays=[], sources=[], samples=[],
            process_sequence=[], other_material=[],
            characteristic_categories=[], comments=[], units=[])
        self.assertEqual(expected_study, self.study)
        self.assertEqual(hash(expected_study),  hash(self.study))

    def test_ne(self):
        expected_other_study = Study(
            filename='file2', identifier='1', title='T2', description='D2',
            submission_date=datetime.datetime(day=2, month=1, year=2017),
            public_release_date=datetime.datetime(day=2, month=1, year=2017),
            contacts=[], design_descriptors=[], publications=[], factors=[],
            protocols=[], assays=[], sources=[], samples=[],
            process_sequence=[], other_material=[],
            characteristic_categories=[], comments=[], units=[])
        self.assertNotEqual(expected_other_study, self.study)
        self.assertNotEqual(hash(expected_other_study), hash(self.study))


class StudyFactorTest(unittest.TestCase):

    def setUp(self):
        self.factor_default = StudyFactor()
        self.factor = StudyFactor(name='N', factor_type=OntologyAnnotation('T'))

    def test_repr(self):
        self.assertEqual("isatools.model.StudyFactor(name='', "
                         "factor_type=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.factor_default))
        self.assertEqual("isatools.model.StudyFactor(name='N', "
                         "factor_type=isatools.model.OntologyAnnotation("
                         "term='T', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.factor))

    def test_str(self):
        self.assertEqual("""StudyFactor(
    name=
    factor_type=
    comments=0 Comment objects
)""", str(self.factor_default))

        self.assertEqual("""StudyFactor(
    name=N
    factor_type=T
    comments=0 Comment objects
)""", str(self.factor))

    def test_eq(self):
        expected_factor = StudyFactor(
            name='N', factor_type=OntologyAnnotation(term='T'))
        self.assertEqual(expected_factor, self.factor)
        self.assertEqual(hash(expected_factor),  hash(self.factor))

    def test_ne(self):
        expected_other_factor = StudyFactor(
            name='N2', factor_type=OntologyAnnotation(term='T2'))
        self.assertNotEqual(expected_other_factor, self.factor)
        self.assertNotEqual(hash(expected_other_factor), hash(self.factor))


class CharacteristicTest(unittest.TestCase):

    def setUp(self):
        self.characteristic_default = Characteristic()
        self.characteristic = Characteristic(
            category=OntologyAnnotation(term='C'), value=0,
            unit=OntologyAnnotation(term='U') )

    def test_repr(self):
        self.assertEqual("isatools.model.Characteristic("
                         "category=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), "
                         "value=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), unit=None, comments=[])",
                         repr(self.characteristic_default))
        self.assertEqual("isatools.model.Characteristic("
                         "category=isatools.model.OntologyAnnotation("
                         "term='C', term_source=None, term_accession='', "
                         "comments=[]), value=0, "
                         "unit=isatools.model.OntologyAnnotation(term='U', "
                         "term_source=None, term_accession='', comments=[]), "
                         "comments=[])", repr(self.characteristic))

    def test_str(self):
        self.assertEqual("""Characteristic(
    category=
    value=
    unit=
    comments=0 Comment objects
)""", str(self.characteristic_default))

        self.assertEqual("""Characteristic(
    category=C
    value=0
    unit=U
    comments=0 Comment objects
)""", str(self.characteristic))

    def test_eq(self):
        expected_characteristic = Characteristic(
            category=OntologyAnnotation(term='C'), value=0,
            unit=OntologyAnnotation(term='U'))
        self.assertEqual(expected_characteristic, self.characteristic)
        self.assertEqual(
            hash(expected_characteristic), hash(self.characteristic))

    def test_ne(self):
        expected_other_characteristic = Characteristic(
            category=OntologyAnnotation(term='C2'), value=1,
            unit=OntologyAnnotation(term='U2'))
        self.assertNotEqual(expected_other_characteristic, self.characteristic)
        self.assertNotEqual(
            hash(expected_other_characteristic), hash(self.characteristic))


class AssayTest(unittest.TestCase):

    def setUp(self):
        self.assay_default = Assay()
        self.assay = Assay(measurement_type=OntologyAnnotation(term='MT'),
                           technology_type=OntologyAnnotation(term='TT'),
                           technology_platform='TP', filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.Assay(measurement_type="
                         "isatools.model.OntologyAnnotation(term='', "
                         "term_source=None, term_accession='', comments=[]), "
                         "technology_type=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), technology_platform='', filename='', "
                         "data_files=[], samples=[], process_sequence=[], "
                         "other_material=[], characteristic_categories=[], "
                         "comments=[], units=[])",
                         repr(self.assay_default))
        self.assertEqual("isatools.model.Assay(measurement_type="
                         "isatools.model.OntologyAnnotation(term='MT', "
                         "term_source=None, term_accession='', comments=[]), "
                         "technology_type=isatools.model.OntologyAnnotation("
                         "term='TT', term_source=None, term_accession='', "
                         "comments=[]), technology_platform='TP', "
                         "filename='file', data_files=[], samples=[], "
                         "process_sequence=[], other_material=[], "
                         "characteristic_categories=[], comments=[], units=[])",
                         repr(self.assay))

    def test_str(self):
        self.assertEqual("""Assay(
    measurement_type=
    technology_type=
    technology_platform=
    filename=
    data_files=0 DataFile objects
    samples=0 Sample objects
    process_sequence=0 Process objects
    other_material=0 Material objects
    characteristic_categories=0 OntologyAnnotation objects
    comments=0 Comment objects
    units=0 Unit objects
)""", str(self.assay_default))

        self.assertEqual("""Assay(
    measurement_type=MT
    technology_type=TT
    technology_platform=TP
    filename=file
    data_files=0 DataFile objects
    samples=0 Sample objects
    process_sequence=0 Process objects
    other_material=0 Material objects
    characteristic_categories=0 OntologyAnnotation objects
    comments=0 Comment objects
    units=0 Unit objects
)""", str(self.assay))

    def test_eq(self):
        expected_assay = Assay(measurement_type=OntologyAnnotation(term='MT'),
                           technology_type=OntologyAnnotation(term='TT'),
                           technology_platform='TP', filename='file')
        self.assertEqual(expected_assay, self.assay)
        self.assertEqual(hash(expected_assay),  hash(self.assay))

    def test_ne(self):
        expected_other_assay = Assay(
            measurement_type=OntologyAnnotation(term='MT2'),
            technology_type=OntologyAnnotation(term='TT2'),
            technology_platform='TP2', filename='file2')
        self.assertNotEqual(expected_other_assay, self.assay)
        self.assertNotEqual(hash(expected_other_assay), hash(self.assay))


class ProtocolTest(unittest.TestCase):

    def setUp(self):
        self.protocol_default = Protocol()
        self.protocol = Protocol(
            name='N', protocol_type=OntologyAnnotation(term='PT'), uri='U',
            version='1')

    def test_repr(self):
        self.assertEqual("isatools.model.Protocol(name='', "
                         "protocol_type=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), uri='', version='', parameters=[], "
                         "components=[], comments=[])",
                         repr(self.protocol_default))
        self.assertEqual("isatools.model.Protocol(name='N', "
                         "protocol_type=isatools.model.OntologyAnnotation("
                         "term='PT', term_source=None, term_accession='', "
                         "comments=[]), uri='U', version='1', parameters=[], "
                         "components=[], comments=[])",
                         repr(self.protocol))

    def test_str(self):
        self.assertEqual("""Protocol(
    name=
    protocol_type=
    uri=
    version=
    parameters=0 ProtocolParameter objects
    components=0 OntologyAnnotation objects
    comments=0 Comment objects
)""", str(self.protocol_default))

        self.assertEqual("""Protocol(
    name=N
    protocol_type=PT
    uri=U
    version=1
    parameters=0 ProtocolParameter objects
    components=0 OntologyAnnotation objects
    comments=0 Comment objects
)""", str(self.protocol))

    def test_eq(self):
        expected_protocol = Protocol(
            name='N', protocol_type=OntologyAnnotation(term='PT'), uri='U',
            version='1')
        self.assertEqual(expected_protocol, self.protocol)
        self.assertEqual(hash(expected_protocol),  hash(self.protocol))

    def test_ne(self):
        expected_other_protocol = Protocol(
            name='N2', protocol_type=OntologyAnnotation(term='PT2'), uri='U2',
            version='2')
        self.assertNotEqual(expected_other_protocol, self.protocol)
        self.assertNotEqual(hash(expected_other_protocol), hash(self.protocol))


class ProtocolParameterTest(unittest.TestCase):

    def setUp(self):
        self.parameter_default = ProtocolParameter()
        self.parameter = ProtocolParameter(
            parameter_name=OntologyAnnotation(term='P'))

    def test_repr(self):
        self.assertEqual("isatools.model.ProtocolParameter("
                         "parameter_name=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.parameter_default))
        self.assertEqual("isatools.model.ProtocolParameter("
                         "parameter_name=isatools.model.OntologyAnnotation("
                         "term='P', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.parameter))

    def test_str(self):
        self.assertEqual("""ProtocolParameter(
    parameter_name=
    comments=0 Comment objects
)""", str(self.parameter_default))

        self.assertEqual("""ProtocolParameter(
    parameter_name=P
    comments=0 Comment objects
)""", str(self.parameter))

    def test_eq(self):
        expected_parameter = ProtocolParameter(
            parameter_name=OntologyAnnotation(term='P'))
        self.assertEqual(expected_parameter, self.parameter)
        self.assertEqual(hash(expected_parameter),  hash(self.parameter))

    def test_ne(self):
        expected_other_parameter = ProtocolParameter(
            parameter_name=OntologyAnnotation(term='P2'))
        self.assertNotEqual(expected_other_parameter, self.parameter)
        self.assertNotEqual(hash(expected_other_parameter), hash(self.parameter))


class ParameterValueTest(unittest.TestCase):

    def setUp(self):
        self.parameter_value_default = ParameterValue()
        self.parameter_value = ParameterValue(
            ProtocolParameter(parameter_name=OntologyAnnotation(term='P')),
            value=0, unit=OntologyAnnotation(term='U'))

    def test_repr(self):
        self.assertEqual("isatools.model.ParameterValue(category=None, "
                         "value=None, unit=None, comments=[])",
                         repr(self.parameter_value_default))
        self.assertEqual("isatools.model.ParameterValue("
                         "category=isatools.model.ProtocolParameter("
                         "parameter_name=isatools.model.OntologyAnnotation("
                         "term='P', term_source=None, term_accession='', "
                         "comments=[]), comments=[]), value=0, "
                         "unit=isatools.model.OntologyAnnotation("
                         "term='U', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.parameter_value))

    def test_str(self):
        self.assertEqual("""ParameterValue(
    category=
    value=None
    unit=
    comments=0 Comment objects
)""", str(self.parameter_value_default))

        self.assertEqual("""ParameterValue(
    category=P
    value=0
    unit=U
    comments=0 Comment objects
)""", str(self.parameter_value))

    def test_eq(self):
        expected_parameter_value = ParameterValue(
            ProtocolParameter(parameter_name=OntologyAnnotation(term='P')),
            value=0, unit=OntologyAnnotation(term='U'))
        self.assertEqual(expected_parameter_value, self.parameter_value)
        self.assertEqual(
            hash(expected_parameter_value),  hash(self.parameter_value))

    def test_ne(self):
        expected_other_parameter_value = ParameterValue(
            ProtocolParameter(parameter_name=OntologyAnnotation(term='P2')),
            value=1, unit=OntologyAnnotation(term='U2'))
        self.assertNotEqual(
            expected_other_parameter_value, self.parameter_value)
        self.assertNotEqual(
            hash(expected_other_parameter_value), hash(self.parameter_value))


class ProtocolComponentTest(unittest.TestCase):

    def setUp(self):
        self.component_default = ProtocolComponent()
        self.component = ProtocolComponent(
            name='C', component_type=OntologyAnnotation(term='CT'))

    def test_repr(self):
        self.assertEqual("isatools.model.ProtocolComponent(name='', "
                         "category=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.component_default))
        self.assertEqual("isatools.model.ProtocolComponent(name='C', "
                         "category=isatools.model.OntologyAnnotation("
                         "term='CT', term_source=None, term_accession='', "
                         "comments=[]), comments=[])",
                         repr(self.component))

    def test_str(self):
        self.assertEqual("""ProtocolComponent(
    name=
    category=
    comments=0 Comment objects
)""", str(self.component_default))

        self.assertEqual("""ProtocolComponent(
    name=C
    category=CT
    comments=0 Comment objects
)""", str(self.component))

    def test_eq(self):
        expected_component = ProtocolComponent(
            name='C', component_type=OntologyAnnotation(term='CT'))
        self.assertEqual(expected_component, self.component)
        self.assertEqual(
            hash(expected_component),  hash(self.component))

    def test_ne(self):
        expected_other_component = ProtocolComponent(
            name='C2', component_type=OntologyAnnotation(term='CT2'))
        self.assertNotEqual(expected_other_component, self.component)
        self.assertNotEqual(
            hash(expected_other_component), hash(self.component))
        

class SourceTest(unittest.TestCase):

    def setUp(self):
        self.source_default = Source()
        self.source = Source(name='S')

    def test_repr(self):
        self.assertEqual("isatools.model.Source(name='', characteristics=[], "
                         "comments=[])",
                         repr(self.source_default))
        self.assertEqual("isatools.model.Source(name='S', characteristics=[], "
                         "comments=[])",
                         repr(self.source))

    def test_str(self):
        self.assertEqual("""Source(
    name=
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.source_default))

        self.assertEqual("""Source(
    name=S
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.source))

    def test_eq(self):
        expected_source = Source(name='S')
        self.assertEqual(expected_source, self.source)
        self.assertEqual(hash(expected_source),  hash(self.source))

    def test_ne(self):
        expected_other_source = Source(name='S2')
        self.assertNotEqual(expected_other_source, self.source)
        self.assertNotEqual(hash(expected_other_source), hash(self.source))


class SampleTest(unittest.TestCase):

    def setUp(self):
        self.sample_default = Sample()
        self.sample = Sample(name='S')

    def test_repr(self):
        self.assertEqual("isatools.model.Sample(name='', characteristics=[], "
                         "factor_values=[], derives_from=[], comments=[])",
                         repr(self.sample_default))
        self.assertEqual("isatools.model.Sample(name='S', characteristics=[], "
                         "factor_values=[], derives_from=[], comments=[])",
                         repr(self.sample))

    def test_str(self):
        self.assertEqual("""Sample(
    name=
    characteristics=0 Characteristic objects
    factor_values=0 FactorValue objects
    derives_from=0 Source objects
    comments=0 Comment objects
)""", str(self.sample_default))

        self.assertEqual("""Sample(
    name=S
    characteristics=0 Characteristic objects
    factor_values=0 FactorValue objects
    derives_from=0 Source objects
    comments=0 Comment objects
)""", str(self.sample))

    def test_eq(self):
        expected_sample = Sample(name='S')
        self.assertEqual(expected_sample, self.sample)
        self.assertEqual(hash(expected_sample),  hash(self.sample))

    def test_ne(self):
        expected_other_sample = Sample(name='S2')
        self.assertNotEqual(expected_other_sample, self.sample)
        self.assertNotEqual(hash(expected_other_sample), hash(self.sample))
        

class ExtractTest(unittest.TestCase):

    def setUp(self):
        self.extract_default = Extract()
        self.extract = Extract(name='E')

    def test_repr(self):
        self.assertEqual("isatools.model.Extract(name='', type='Extract Name', "
                         "characteristics=[], comments=[])",
                         repr(self.extract_default))
        self.assertEqual("isatools.model.Extract(name='E', "
                         "type='Extract Name', characteristics=[], "
                         "comments=[])",
                         repr(self.extract))

    def test_str(self):
        self.assertEqual("""Extract(
    name=
    type=Extract Name
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.extract_default))

        self.assertEqual("""Extract(
    name=E
    type=Extract Name
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.extract))

    def test_eq(self):
        expected_extract = Extract(name='E')
        self.assertEqual(expected_extract, self.extract)
        self.assertEqual(hash(expected_extract),  hash(self.extract))

    def test_ne(self):
        expected_other_extract = Extract(name='S2')
        self.assertNotEqual(expected_other_extract, self.extract)
        self.assertNotEqual(hash(expected_other_extract), hash(self.extract))


class LabeledExtractTest(unittest.TestCase):

    def setUp(self):
        self.labeled_extract_default = LabeledExtract()
        self.labeled_extract = LabeledExtract(name='E')

    def test_repr(self):
        self.assertEqual("isatools.model.LabeledExtract(name='', "
                         "type='Labeled Extract Name', characteristics=[], "
                         "comments=[])", repr(self.labeled_extract_default))
        self.assertEqual("isatools.model.LabeledExtract(name='E', "
                         "type='Labeled Extract Name', characteristics=[], "
                         "comments=[])", repr(self.labeled_extract))

    def test_str(self):
        self.assertEqual("""LabeledExtract(
    name=
    type=LabeledExtract Name
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.labeled_extract_default))

        self.assertEqual("""LabeledExtract(
    name=E
    type=LabeledExtract Name
    characteristics=0 Characteristic objects
    comments=0 Comment objects
)""", str(self.labeled_extract))

    def test_eq(self):
        expected_labeled_extract = LabeledExtract(name='E')
        self.assertEqual(expected_labeled_extract, self.labeled_extract)
        self.assertEqual(hash(expected_labeled_extract),
                         hash(self.labeled_extract))

    def test_ne(self):
        expected_other_labeled_extract = LabeledExtract(name='S2')
        self.assertNotEqual(expected_other_labeled_extract, self.labeled_extract)
        self.assertNotEqual(hash(expected_other_labeled_extract),
                            hash(self.labeled_extract))


class FactorValueTest(unittest.TestCase):

    def setUp(self):
        self.factor_value_default = FactorValue()
        self.factor_value = FactorValue(factor_name=StudyFactor(name='F'),
            value=0, unit=OntologyAnnotation(term='U'))

    def test_repr(self):
        self.assertEqual("isatools.model.FactorValue(factor_name=None, "
                         "value=None, unit=None)",
                         repr(self.factor_value_default))
        self.assertEqual("isatools.model.FactorValue("
                         "factor_name=isatools.model.StudyFactor(name='F', "
                         "factor_type=isatools.model.OntologyAnnotation("
                         "term='', term_source=None, term_accession='', "
                         "comments=[]), comments=[]), value=0, "
                         "unit=isatools.model.OntologyAnnotation(term='U', "
                         "term_source=None, term_accession='', comments=[]))",
                         repr(self.factor_value))

    def test_str(self):
        self.assertEqual("""FactorValue(
    factor_name=
    value=None
    unit=
)""", str(self.factor_value_default))

        self.assertEqual("""FactorValue(
    factor_name=F
    value=0
    unit=U
)""", str(self.factor_value))

    def test_eq(self):
        expected_factor_value = FactorValue(factor_name=StudyFactor(name='F'),
            value=0, unit=OntologyAnnotation(term='U'))
        self.assertEqual(expected_factor_value, self.factor_value)
        self.assertEqual(
            hash(expected_factor_value),  hash(self.factor_value))

    def test_ne(self):
        expected_other_factor_value = FactorValue(
            factor_name=StudyFactor(name='F2'), value=1,
            unit=OntologyAnnotation(term='U2'))
        self.assertNotEqual(
            expected_other_factor_value, self.factor_value)
        self.assertNotEqual(
            hash(expected_other_factor_value), hash(self.factor_value))


class DataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = DataFile()
        self.data_file = DataFile(filename='file', label='Data File Name')

    def test_repr(self):
        self.assertEqual("isatools.model.DataFile(filename='', label='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.DataFile(filename='file', "
                         "label='Data File Name', generated_from=[], "
                         "comments=[])", repr(self.data_file))

    def test_str(self):
        self.assertEqual("""DataFile(
    filename=
    label=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""DataFile(
    filename=file
    label=Data File Name
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = DataFile(filename='file', label='Data File Name')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = DataFile(filename='file2',
                                            label='Raw Data File')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file), hash(self.data_file))


class RawDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = RawDataFile()
        self.data_file = RawDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.RawDataFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.RawDataFile(filename='file', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""RawDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""RawDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = RawDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = RawDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        
        
class DerivedDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = DerivedDataFile()
        self.data_file = DerivedDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.DerivedDataFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.DerivedDataFile(filename='file', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""DerivedDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""DerivedDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = DerivedDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = DerivedDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class RawSpectralDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = RawSpectralDataFile()
        self.data_file = RawSpectralDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.RawSpectralDataFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.RawSpectralDataFile(filename='file', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""RawSpectralDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""RawSpectralDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = RawSpectralDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = RawSpectralDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class ArrayDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = ArrayDataFile()
        self.data_file = ArrayDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.ArrayDataFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.ArrayDataFile(filename='file', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""ArrayDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""ArrayDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = ArrayDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = ArrayDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class DerivedSpectralDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = DerivedSpectralDataFile()
        self.data_file = DerivedSpectralDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.DerivedSpectralDataFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.DerivedSpectralDataFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""DerivedSpectralDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""DerivedSpectralDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = DerivedSpectralDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = DerivedSpectralDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        
        
class ProteinAssignmentFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = ProteinAssignmentFile()
        self.data_file = ProteinAssignmentFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.ProteinAssignmentFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.ProteinAssignmentFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""ProteinAssignmentFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""ProteinAssignmentFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = ProteinAssignmentFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = ProteinAssignmentFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        
        
class PeptideAssignmentFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = PeptideAssignmentFile()
        self.data_file = PeptideAssignmentFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.PeptideAssignmentFile(filename='', "
                         "generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.PeptideAssignmentFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""PeptideAssignmentFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""PeptideAssignmentFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = PeptideAssignmentFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = PeptideAssignmentFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class DerivedArrayDataMatrixFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = DerivedArrayDataMatrixFile()
        self.data_file = DerivedArrayDataMatrixFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.DerivedArrayDataMatrixFile("
                         "filename='', generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.DerivedArrayDataMatrixFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""DerivedArrayDataMatrixFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""DerivedArrayDataMatrixFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = DerivedArrayDataMatrixFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = DerivedArrayDataMatrixFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class PostTranslationalModificationAssignmentFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = PostTranslationalModificationAssignmentFile()
        self.data_file = PostTranslationalModificationAssignmentFile(
            filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model."
                         "PostTranslationalModificationAssignmentFile("
                         "filename='', generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model."
                         "PostTranslationalModificationAssignmentFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""PostTranslationalModificationAssignmentFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""PostTranslationalModificationAssignmentFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = PostTranslationalModificationAssignmentFile(
            filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = PostTranslationalModificationAssignmentFile(
            filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class AcquisitionParameterDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = AcquisitionParameterDataFile()
        self.data_file = AcquisitionParameterDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.AcquisitionParameterDataFile("
                         "filename='', generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.AcquisitionParameterDataFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""AcquisitionParameterDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""AcquisitionParameterDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = AcquisitionParameterDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = AcquisitionParameterDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))
        

class FreeInductionDecayDataFileTest(unittest.TestCase):

    def setUp(self):
        self.data_file_default = FreeInductionDecayDataFile()
        self.data_file = FreeInductionDecayDataFile(filename='file')

    def test_repr(self):
        self.assertEqual("isatools.model.FreeInductionDecayDataFile("
                         "filename='', generated_from=[], comments=[])",
                         repr(self.data_file_default))
        self.assertEqual("isatools.model.FreeInductionDecayDataFile("
                         "filename='file', generated_from=[], comments=[])",
                         repr(self.data_file))

    def test_str(self):
        self.assertEqual("""FreeInductionDecayDataFile(
    filename=
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file_default))

        self.assertEqual("""FreeInductionDecayDataFile(
    filename=file
    generated_from=0 Sample objects
    comments=0 Comment objects
)""", str(self.data_file))

    def test_eq(self):
        expected_data_file = FreeInductionDecayDataFile(filename='file')
        self.assertEqual(expected_data_file, self.data_file)
        self.assertEqual(hash(expected_data_file),  hash(self.data_file))

    def test_ne(self):
        expected_other_data_file = FreeInductionDecayDataFile(filename='file2')
        self.assertNotEqual(expected_other_data_file, self.data_file)
        self.assertNotEqual(hash(expected_other_data_file),
                            hash(self.data_file))


from __future__ import absolute_import

import os
import logging
import numpy as np
import pandas as pd


from isatools.isatab_meta import AbstractParser, InvestigationParser, TableParser
from isatools.model import *


log = logging.getLogger(__name__)
__author__ = 'djcomlab@gmail.com (David Johnson)'


class IsaTabSeries(pd.Series):
    @property
    def _constructor(self):
        return IsaTabSeries


class IsaTabDataFrame(pd.DataFrame):

    def __init__(self, *args, **kw):
        super(IsaTabDataFrame, self).__init__(*args, **kw)

    @property
    def _constructor(self):
        return IsaTabDataFrame

    _constructor_sliced = IsaTabSeries

    @staticmethod
    def _clean_label(label):
        for clean_label in TableParser.ALL_LABELS:
            if label.strip().lower().startswith(clean_label.lower()):
                return clean_label
            elif TableParser.RX_CHARACTERISTICS.match(label):
                return 'Characteristics[{val}]'.format(
                    val=next(iter(
                        TableParser.RX_CHARACTERISTICS.findall(label))))
            elif TableParser.RX_PARAMETER_VALUE.match(label):
                return 'Parameter Value[{val}]'.format(
                    val=next(iter(
                        TableParser.RX_PARAMETER_VALUE.findall(label))))
            elif TableParser.RX_FACTOR_VALUE.match(label):
                return 'Factor Value[{val}]'.format(
                    val=next(iter(TableParser.RX_FACTOR_VALUE.findall(label))))
            elif TableParser.RX_COMMENT.match(label):
                return 'Comment[{val}]'.format(
                    val=next(iter(TableParser.RX_COMMENT.findall(label))))

    @property
    def isatab_header(self):
        return list(map(lambda x: self._clean_label(x), self.columns))


class StudySampleTableParser(TableParser):

    def __init__(self, isa=None):
        TableParser.__init__(self)
        if not isinstance(isa, Investigation):
            raise IOError('You must provide an Investigation object output '
                          'from the Investigation parser')
        self.isa = isa
        self.sources = []
        self.samples = []
        self.characteristic_categories = dict()
        self.unit_categories = dict()
        self.process_sequence = []

    def _parse_object_characteristics(self, labels, row):
        characteristics = set()
        for base_column in [x for x in labels if x.startswith(
                ('Characteristics', 'Material Type'))]:
            if base_column.startswith('Material Type'):
                category_key = 'Material Type'
            else:
                category_key = base_column[16:-1]
            try:
                category = self.characteristic_categories[
                    category_key]
            except KeyError:
                category = OntologyAnnotation(term=category_key)
                self.characteristic_categories[category_key] = category
            characteristic = Characteristic(category=category)
            value, unit = self._get_value(base_column, labels, row)
            characteristic.value = value
            characteristic.unit = unit
            characteristics.add(characteristic)
        return sorted(list(characteristics), key=lambda x: x.category.term)

    def _parse_object_comments(self, labels, row):
        comments = set()
        for base_column in (x for x in labels if x.startswith('Comment')):
            comment = Comment(name=base_column[8:-1])
            value, unit = self._get_value(base_column, labels, row)
            comment.value = value
            comment.unit = unit
            comments.add(comment)
        return sorted(list(comments), key=lambda x: x.name)

    def _parse_object_factor_values(self, labels, row):
        factor_values = set()
        for base_column in (x for x in labels if x.startswith('Factor Value')):
            factor_value = FactorValue(factor_name=StudyFactor(
                name=base_column[13:-1]))  # TODO: Check is in Study obj
            value, unit = self._get_value(base_column, labels, row)
            factor_value.value = value
            factor_value.unit = unit
            factor_values.add(factor_value)
        return sorted(list(factor_values), key=lambda x: x.factor_name.name)

    def _parse_materials(self, material_df):
        materials = []
        for _, row in material_df.drop_duplicates().iterrows():
            material_label = next(iter(material_df.columns))
            if material_label.startswith('Source Name'):
                material = Source(name=row[material_label])
            elif material_label.startswith('Sample Name'):
                material = Sample(name=row[material_label])
            material.characteristics = self._parse_object_characteristics(
                material_df.columns, row)
            material.comments = self._parse_object_comments(
                material_df.columns, row)
            material.factor_values = self._parse_object_factor_values(
                material_df.columns, row)
            materials.append(material)
        materials_dict = dict(
            map(lambda x: ('.'.join([material_label, x.name]), x), materials))
        return materials_dict

    def _parse_data_files(self, data_file_df):
        data_files = []
        for _, row in data_file_df.drop_duplicates().iterrows():
            data_file_label = next(iter(data_file_df.columns))
            if data_file_label.startswith('Raw Data File'):
                data_file = RawDataFile(filename=row[data_file_label])
            elif data_file_label.startswith('Derived Data File'):
                data_file = DerivedDataFile(filename=row[data_file_label])
            elif data_file_label.startswith('Derived Spectral Data File'):
                data_file = DerivedSpectralDataFile(
                    filename=row[data_file_label])
            elif data_file_label.startswith('Derived Array Data File'):
                data_file = DerivedArrayDataFile(filename=row[data_file_label])
            elif data_file_label.startswith('Array Data File'):
                data_file = ArrayDataFile(filename=row[data_file_label])
            elif data_file_label.startswith('Protein Assignment File'):
                data_file = ProteinAssignmentFile(filename=row[data_file_label])
            elif data_file_label.startswith('Peptide Assignment File'):
                data_file = PeptideAssignmentFile(filename=row[data_file_label])
            elif data_file_label.startswith(
                    'Post Translational Modification Assignment File'):
                data_file = PostTranslationalModificationAssignmentFile(
                    filename=row[data_file_label])
            elif data_file_label.startswith('Acquisition Parameter Data File'):
                data_file = AcquisitionParameterDataFile(
                    filename=row[data_file_label])
            elif data_file_label.startswith('Free Induction Decay Data File'):
                data_file = FreeInductionDecayDataFile(
                    filename=row[data_file_label])
            # elif data_file_label.startswith('Image File'):
            #     data_file = ImageFile(filename=row[data_file_label])
            # elif data_file_label.startswith('Metabolite Assignment File'):
            #     data_file = MetaboliteAssignmentFile(filename=row[data_file_label])
            elif data_file_label.startswith('Raw Spectral Data File'):
                data_file = RawSpectralDataFile(filename=row[data_file_label])
            data_file.comments = self._parse_object_comments(
                data_file_df.columns, row)
            data_files.append(data_file)
        data_files_dict = dict(
            map(lambda x: ('.'.join([data_file_label, x.name]), x), data_files))
        return data_files_dict

    def _parse_protocol_ref(self, protocol_ref_df):
        pass

    def _parse(self, filebuffer):
        isa_df = IsaTabDataFrame(
            pd.read_csv(filebuffer, dtype=str, sep='\t', encoding='utf-8',
                        comment='#').replace(np.nan, ''))
        isatab_header = isa_df.isatab_header
        object_index = [
            i for i, x in enumerate(isatab_header) if
            x in TableParser.MATERIAL_LABELS + TableParser.DATA_FILE_LABELS +
            tuple(['Protocol REF'])]
        object_columns_grouped = []
        prev_i = next(iter(object_index))
        for curr_i in object_index:
            if prev_i != curr_i:
                object_columns_grouped.append(isa_df.columns[prev_i:curr_i])
            prev_i = curr_i
            object_columns_grouped.append(isa_df.columns[prev_i:])
        for object_column_group in object_columns_grouped:
            object_label = next(iter(object_column_group))  # indicates object
            chopped_isa_df = isa_df[object_column_group]  # chopped is cut by column. sliced is cut by row
            if object_label.startswith('Source Name'):
                sources = self._parse_materials(chopped_isa_df)
                self.node_map.update(sources)
            elif object_label.startswith('Sample Name'):
                samples = self._parse_materials(chopped_isa_df)
                self.node_map.update(samples)
        self.sources = [v for k, v in self.node_map.items() if
                        k.startswith('Source Name')]
        self.samples = [v for k, v in self.node_map.items() if
                        k.startswith('Sample Name')]
        self._make_process_sequence(isa_df)
        self.process_sequence = list(self.process_map.values())


class AssayTableParser(TableParser):

    def __init__(self, isa=None):
        TableParser.__init__(self)
        if not isinstance(isa, Investigation):
            raise IOError('You must provide an Investigation object output '
                          'from the Investigation parser')
        self.isa = isa
        self.samples = None
        self.other_material = None
        self.data_files = None
        self.node_map = dict()
        self.process_map = dict()
        self.process_sequence = None

    def _parse(self, filebuffer):
        df = pd.read_csv(filebuffer, dtype=str, sep='\t', encoding='utf-8',
                         comment='#').replace(np.nan, '')
        samples = dict(
            map(lambda x: ('.'.join(['Sample Name', x]), Sample(name=x)),
                [str(x) for x in df['Sample Name'].drop_duplicates()
                 if x != '']))

        data_files = dict()
        for data_col in (x for x in df.columns if
                         x in TableParser.DATA_FILE_LABELS):
            filenames = [x for x in df[data_col].drop_duplicates() if x != '']
            data_files.update(
                dict(map(
                    lambda x: ('.'.join([data_col, x]),
                               DataFile(filename=x, label=data_col)),
                    filenames)))

        other_material = dict()
        for material_col in (x for x in df.columns if
                             x in TableParser.OTHER_MATERIAL_LABELS):
            if material_col == 'Extract Name':
                extracts = dict(
                    map(lambda x: (
                        '.'.join(['Extract Name', x]), Extract(name=x)),
                        [str(x) for x in df['Extract Name'].drop_duplicates()
                         if x != '']))
                other_material.update(extracts)
            elif material_col == 'Labeled Extract Name':
                labeled_extracts = dict(
                    map(lambda x: (
                        '.'.join(['Labeled Extract Name', x]),
                        LabeledExtract(name=x)),
                        [str(x) for x in
                         df['Labeled Extract Name'].drop_duplicates()
                         if x != '']))
                other_material.update(labeled_extracts)

        self.samples = list(samples.values())
        self.node_map.update(samples)
        self.data_files = list(data_files.values())
        self.node_map.update(data_files)
        self.other_material = list(other_material.values())
        self.node_map.update(other_material)
        self._make_process_sequence(df)
        self.process_sequence = list(self.process_map.values())


class Parser(AbstractParser):

    def __init__(self):
        self.investigation_parser = InvestigationParser()

    def _parse(self, filebuffer):
        self.investigation_parser.parse(filebuffer)

        for study in self.investigation_parser.isa.studies:
            study_sample_table_parser = StudySampleTableParser(
                self.investigation_parser.isa)
            study_sample_table_parser.parse(
                os.path.join(os.path.dirname(filebuffer.name), study.filename))
            study.sources = study_sample_table_parser.sources
            study.samples = study_sample_table_parser.samples
            study.process_sequence = study_sample_table_parser.process_sequence
            for assay in study.assays:
                assay_table_parser = AssayTableParser(
                    self.investigation_parser.isa)
                assay_table_parser.parse(
                    os.path.join(os.path.dirname(filebuffer.name),
                                 assay.filename))
                assay.samples = assay_table_parser.samples
                assay.data_files = assay_table_parser.data_files
                assay.other_material = assay_table_parser.other_material
                assay.process_sequence = assay_table_parser.process_sequence
        self.isa = self.investigation_parser.isa



from __future__ import absolute_import
import io
import os
import shutil
import tempfile
import unittest

from isatools import isatab
from isatools import isaviz


class TestIsaVizualization(unittest.TestCase):

    def setUp(self):
        self.tmp_dir = tempfile.mkdtemp()
        self._tab_data_dir = os.path.join(os.path.dirname(__file__), 'data',
                                          'tab')
        self.parser = isatab.Parser()
        with io.open(os.path.join(self._tab_data_dir, 'BII-I-1',
                                  'i_investigation.txt')) as fp:
            self.parser.parse(fp)

        self.study = next(iter(self.parser.isa.studies))
        self.assay = next(iter(self.study.assays))

    def tearDown(self):
        shutil.rmtree(self.tmp_dir)

    def test_make_study_summary(self):
        isaviz.make_study_summary(self.study, self.tmp_dir)
        self.assertTrue(
            os.path.isfile(os.path.join(self.tmp_dir,
                                        '{}.png'.format(next(iter(
                                            os.path.splitext(
                                                self.study.filename)))))))

    def test_make_assay_summary(self):
        isaviz.make_assay_summary(self.assay, self.tmp_dir)
        self.assertTrue(
            os.path.isfile(os.path.join(self.tmp_dir,
                                        '{}.png'.format(next(iter(
                                            os.path.splitext(
                                                self.assay.filename)))))))


#!/usr/bin/env python

from setuptools import setup

setup(
    name='isa-rwval',
    version='0.10.9',
    packages=['isatools',
              'isatools.errors'
              ],
    package_data={'isatools': [],
                  '': ['LICENSE.txt', 'README.md']},
    description='Metadata tracking tools help to manage an increasingly '
                'diverse set of life science, environmental and biomedical '
                'experiments',
    author='ISA Infrastructure Team',
    author_email='isatools@googlegroups.com',
    url='https://github.com/ISA-tools/isa-rwval',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Topic :: Scientific/Engineering :: Bio-Informatics',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6'
        ],
    install_requires=[
        'networkx==1.11',
        'six>=1.11.0',
    ],
    extras_require={
        'viz': ['matplotlib'],
        'tables': ['pandas', 'numpy']
    },
    test_suite='tests'
)
